import { api } from "dicomweb-client";
import dcmjs from "dcmjs";
import { utilities } from "@cornerstonejs/core";
import cornerstoneDICOMImageLoader from "@cornerstonejs/dicom-image-loader";
import { getPixelSpacingInformation } from "./getPixelSpacingInformation";
import { convertMultiframeImageIds } from "./convertMultiframeImageIds";
import { removeInvalidTags } from "./removeInvalidTags";
import getPTImageIdInstanceMetadata from "./getPTImageIdInstanceMetadata";
import { calculateSUVScalingFactors } from "@cornerstonejs/calculate-suv";
import ptScalingMetaDataProvider from "./ptScalingMetaDataProvider";

const { DicomMetaDictionary } = dcmjs.data;
const { calibratedPixelSpacingMetadataProvider } = utilities;

const SOP_INSTANCE_UID = "00080018";
const SERIES_INSTANCE_UID = "0020000E";
const MODALITY = "00080060";

interface CreateImageIdsAndCacheMetaDataOptions {
  StudyInstanceUID: string;
  SeriesInstanceUID: string;
  SOPInstanceUID?: string | null;
  wadoRsRoot: string;
  client?: api.DICOMwebClient | null;
}

/**
 * 从 DICOMweb 服务创建图像 ID 并缓存相关元数据。
 *
 * @param options - 包含图像和 DICOMweb 服务信息的对象
 * @returns 图像 ID 数组
 */
export const createImageIdsAndCacheMetaData = async (
  options: CreateImageIdsAndCacheMetaDataOptions
): Promise<string[]> => {
  const client =
    options.client ||
    new api.DICOMwebClient({ url: options.wadoRsRoot, singlepart: true });

  try {
    const instances = await client.retrieveSeriesMetadata({
      studyInstanceUID: options.StudyInstanceUID,
      seriesInstanceUID: options.SeriesInstanceUID,
    });

    const modality = (instances[0] as unknown as any)[MODALITY].Value[0];
    console.log("modality", modality);

    let imageIds = instances.map((instanceMetaData: any) => {
      const SeriesInstanceUID = instanceMetaData[SERIES_INSTANCE_UID].Value[0];
      const SOPInstanceUIDToUse =
        options.SOPInstanceUID || instanceMetaData[SOP_INSTANCE_UID].Value[0];

      const prefix = "wadors:";

      const imageId =
        prefix +
        options.wadoRsRoot +
        "/studies/" +
        options.StudyInstanceUID +
        "/series/" +
        SeriesInstanceUID +
        "/instances/" +
        SOPInstanceUIDToUse +
        "/frames/1";

      cornerstoneDICOMImageLoader.wadors.metaDataManager.add(
        imageId,
        instanceMetaData
      );
      return imageId;
    });

    imageIds = convertMultiframeImageIds(imageIds);

    imageIds.forEach((imageId) => {
      let instanceMetaData =
        cornerstoneDICOMImageLoader.wadors.metaDataManager.get(imageId);

      instanceMetaData = removeInvalidTags(instanceMetaData);

      if (instanceMetaData) {
        const metadata =
          DicomMetaDictionary.naturalizeDataset(instanceMetaData);
        const pixelSpacing = getPixelSpacingInformation(metadata) as Number[];

        if (pixelSpacing) {
          // FIXME: cornerstone类型定义有问题，这里.add方法缺少type属性
          calibratedPixelSpacingMetadataProvider.add(imageId, {
            // @ts-ignore
            rowPixelSpacing: pixelSpacing[0],
            // @ts-ignore
            columnPixelSpacing: pixelSpacing[1],
          });
        }
      }
    });

    if (modality === "PT") {
      const InstanceMetadataArray: any = [];
      imageIds.forEach((imageId) => {
        const instanceMetadata = getPTImageIdInstanceMetadata(imageId);

        // TODO: Temporary fix because static-wado is producing a string, not an array of values
        // (or maybe dcmjs isn't parsing it correctly?)
        // It's showing up like 'DECY\\ATTN\\SCAT\\DTIM\\RAN\\RADL\\DCAL\\SLSENS\\NORM'
        // but calculate-suv expects ['DECY', 'ATTN', ...]
        if (typeof instanceMetadata.CorrectedImage === "string") {
          instanceMetadata.CorrectedImage =
            instanceMetadata.CorrectedImage.split("\\");
        }

        if (instanceMetadata) {
          InstanceMetadataArray.push(instanceMetadata);
        }
      });
      if (InstanceMetadataArray.length) {
        try {
          const suvScalingFactors = calculateSUVScalingFactors(
            InstanceMetadataArray
          );
          InstanceMetadataArray.forEach((instanceMetadata, index) => {
            ptScalingMetaDataProvider.addInstance(
              imageIds[index],
              suvScalingFactors[index]
            );
          });
        } catch (error) {
          console.log(error);
        }
      }
    }

    return imageIds;
  } catch (error) {
    console.log(error);
    throw new Error("PACS 中数据不存在");
  }
};
