import { relative } from "path";
import React, { useEffect, useRef, useState } from "react";
import * as THREE from "three";
import { TrackballControls } from "three/examples/jsm/controls/TrackballControls";

type ThreeJSComponentProps = {
  // 你可以在这里添加任何需要的props
};

export const MprViewer: React.FC<ThreeJSComponentProps> = () => {
  const mountRef = useRef<HTMLDivElement>(null);
  const [imageSrc, setImageSrc] = useState<string>("");

  useEffect(() => {
    const width = window.innerWidth - 300;
    const height = window.innerHeight;
    const scene = new THREE.Scene();
    scene.background = new THREE.Color(0x808080);

    const camera = new THREE.PerspectiveCamera(75, width / height, 0.1, 1000);
    const renderer = new THREE.WebGLRenderer({ antialias: true });
    renderer.setSize(width, height);

    // 添加一个可移动和旋转的平面
    const planeGeometry = new THREE.PlaneGeometry(200, 200);
    const planeMaterial = new THREE.MeshBasicMaterial({
      color: 0xffff00,
      side: THREE.DoubleSide,
    });
    const plane = new THREE.Mesh(planeGeometry, planeMaterial);
    plane.position.set(0, 0, 0); // 初始位置
    scene.add(plane);

    // 创建一个与平面对齐的剪裁平面
    const clipPlane = new THREE.Plane(new THREE.Vector3(0, 0, 1), 0);
    renderer.clippingPlanes = [clipPlane];

    // 使剪裁平面跟随你的移动平面
    const updateClipPlane = () => {
      clipPlane.normal.copy(plane.position).normalize();
    };

    // 添加多个3D物体
    const geometry1 = new THREE.BoxGeometry(100, 100, 100);
    const material1 = new THREE.MeshPhongMaterial({
      color: 0x00ff00,
      transparent: true,
      opacity: 0.5,
    });
    const cube = new THREE.Mesh(geometry1, material1);
    cube.position.x = -50; // 调整位置以确保重叠
    scene.add(cube);

    const geometry2 = new THREE.SphereGeometry(50, 32, 32);
    const material2 = new THREE.MeshPhongMaterial({
      color: 0xff0000,
      transparent: true,
      opacity: 0.5,
    });
    const sphere = new THREE.Mesh(geometry2, material2);
    sphere.position.x = 0; // 调整位置以确保重叠
    scene.add(sphere);

    const geometry3 = new THREE.ConeGeometry(50, 100, 32);
    const material3 = new THREE.MeshPhongMaterial({
      color: 0x0000ff,
      transparent: true,
      opacity: 0.5,
    });
    const cone = new THREE.Mesh(geometry3, material3);
    cone.position.x = 50; // 调整位置以确保重叠
    scene.add(cone);

    camera.position.set(0, 0, 500);
    camera.lookAt(scene.position);

    const ambientLight = new THREE.AmbientLight(0xffffff, 0.5);
    scene.add(ambientLight);

    const mount = mountRef.current;
    if (mount) {
      mount.appendChild(renderer.domElement);
    }

    const controls = new TrackballControls(camera, renderer.domElement);

    const animate = () => {
      requestAnimationFrame(animate);
      controls.update(); // 只有在使用 TrackballControls 时才需要
      // camera.position.y += 5;
      camera.updateMatrixWorld();

      updateClipPlane(); // 更新剪裁平面的位置和方向


      renderer.render(scene, camera);

      // 从渲染器的 canvas 中获取图像
      const canvas = renderer.domElement;
      canvas.toBlob((blob) => {
        if (blob) {
          const imageUrl = URL.createObjectURL(blob);
          setImageSrc(imageUrl);
        }
      }, "image/jpeg");
    };

    animate();

    return () => {
      controls.dispose();
      mount?.removeChild(renderer.domElement);
      URL.revokeObjectURL(imageSrc);
    };
  }, []);

  return (
    <div style={{ position: "relative" }}>
      <div ref={mountRef} />
      <img
        style={{
          position: "absolute",
          right: 0,
          top: 0,
          width: "200px",
          height: "200px",
        }}
        src={imageSrc}
        alt="Rendered Scene"
      />
    </div>
  );
};
