import { User } from "@@/domain/User/entities/User";
import { Request } from "./Request";
import axios from "axios";
import { saveAs } from "file-saver";
import {
  ArchiveTaskCreateDto,
  DownloadArchiveDTO,
  ExistInPacsDTO,
  LabelCreateDTO,
  LabelDeleteDTO,
  LabelUpdateDTO,
  ResponsePacsType,
  ResponseType,
  labelCategoryDTO,
} from "./dto";
import { Label } from "@@/domain/Label/entities/label";

const PREFIX = "/api/dmp";
const PREFIX_CERT = "/cert";
const PREFIX_PACS = "/dicom-web";

export const Apis = {
  /**
   * 用户登录
   */
  signIn: (p: Pick<User, "username" | "password">): ResponseType =>
    Request.post(PREFIX + "/auth/signIn", p),
  /**
   * 用户认证
   */
  userAuth: (): ResponseType => Request.get(PREFIX_CERT + "/auth/user"),
  /**
   * 上传dcm到pacs
   */
  upload2Pacs: (fd: FormData): ResponsePacsType =>
    Request.post(PREFIX_PACS + "/instances", fd, {
      headers: {
        "Content-Type": "multipart/form-data",
      },
    }),
  /**
   * pacs当中是否存在该序列，返回切片数量对比上传的File[].length
   */
  existInPacs: ({
    StudyInstanceUID,
    SeriesInstanceUID,
  }: ExistInPacsDTO): Promise<[]> =>
    Request.get(
      PREFIX_PACS +
        `/dicom-web/studies/${StudyInstanceUID}/series/${SeriesInstanceUID}/instances`
    ),
  /**
   * 获取标注人员信息
   */
  getDmpAnnotators: (): ResponseType =>
    Request.get(PREFIX + `/admin/find/annotator`),

  /**
   * 创建标注任务
   */
  createArchiveTask: (p: ArchiveTaskCreateDto): ResponseType =>
    Request.post(PREFIX + "/admin/create/archiveTask", p),

  /**
   * 查询归档任务
   */
  findArchiveTask: (): ResponseType =>
    Request.get(PREFIX + "/annotator/find/archiveTask"),

  findDicoms: (): ResponseType => Request.get(PREFIX + "/admin/find/dicom/all"),

  /**
   * 下载dicom zip
   *
   */
  downloadDicom: (p: DownloadArchiveDTO) =>
    axios
      .post(PREFIX + `/dicom/download`, p, {
        responseType: "blob",
      })
      .then((response) => {
        const filename = response.headers["content-name"] as string;
        const blob = new Blob([response.data], { type: "application/zip" });
        saveAs(blob, filename);
      })
      .catch((error) => {
        console.error("Download error:", error);
      }),
  /**
   * 创建新的标签分类
   */
  createNewLabelCategory: (p: labelCategoryDTO): ResponseType =>
    Request.post(PREFIX + "/dicom/label/category/create", p),

  /**
   * 查询全部标签分类
   */
  findLabelCategory: (): ResponseType =>
    Request.get(PREFIX + "/dicom/label/category/find/all"),

  /**
   * 创建新标签
   */
  createLabel: (p: LabelCreateDTO): ResponseType =>
    Request.post(PREFIX + "/dicom/label/create", p),

  /**
   * 修改标签
   */
  updateLabel: (p: LabelUpdateDTO): ResponseType =>
    Request.post(PREFIX + "/dicom/label/update", p),

  /**
   * 删除标签
   */
  deleteLabel: (p: LabelDeleteDTO): ResponseType =>
    Request.post(PREFIX + "/dicom/label/delete", p),

  /**
   * 修改标签分类
   */
  updateCategory: (p: labelCategoryDTO): ResponseType =>
    Request.post(PREFIX + "/dicom/label/category/update", p),
};
